@@ -4,75 +4,119 @@ use 5.010;
 use strict;
 use warnings;
 
-use base 'Csistck::Test::FileBase';
-use Csistck::Oper qw/debug/;
-use Csistck::Util qw/hash_file hash_string/;
-
+use base 'Exporter';
 our @EXPORT_OK = qw/template/;
 
+use Csistck::Oper qw/debug/;
+use Csistck::Util qw/backup_file hash_file hash_string/;
+use Csistck::Test;
+
 use Template;
 use File::Copy;
 use Sys::Hostname::Long qw//;
 use FindBin;
 use Text::Diff ();
 
-sub template { Csistck::Test::Template->new(@_); };
+sub template {
+    my $template = shift;
+    my $dest = shift;
+    my $args = shift // {};
 
-sub desc { sprintf("Template check for destination %s", shift->dest); }
+    # Die on invalid arguments
+    die("Invalid template name")
+      unless($template =~ /^[A-Za-z0-9\-\_][A-Za-z0-9\/\-\_\.]+$/);
+    die("Destination not specified")
+      unless(defined $dest);
+    
+    # Get full template path, return Test
+    my $abs_tpl = get_absolute_template($template);
 
-sub file_check {
-    my $self = shift;
+    # Add automatic arguments
+    my $args_add = {
+        hostname => Sys::Hostname::Long::hostname_long(),
+        %{$args}
+    };
+
+    return Csistck::Test->new(
+      check => sub { template_check($abs_tpl, $dest, $args_add); },
+      repair => sub { template_install($abs_tpl, $dest, $args_add); },
+      diff => sub { template_diff($abs_tpl, $dest, $args_add); },
+      desc => "Process template $template for destination $dest"
+    );
+}
+
+sub template_check {
+    my ($template, $dest, $args) = @_;
     my $tplout;
 
-    $self->template_file(\$tplout)
-      or die("Template file not processed: template=<${\$self->src}>");
+    template_file($template, \$tplout, $args)
+      or die("Template file not processed: template=<$template>");
     
     my $hashsrc = hash_string($tplout);
-    my $hashdst = hash_file($self->dest);
+    my $hashdst = hash_file($dest);
     
-    return (defined $hashsrc and defined $hashdst and ($hashsrc eq $hashdst));
+    die("Template output does not match destination")
+      unless(defined $hashsrc and defined $hashdst and ($hashsrc eq $hashdst));
 }
 
-sub file_repair {
-    my $self = shift;
-    debug(sprintf("Output template: template=<%s> dest=<%s>",
-      $self->src, $self->dest));
-    # TODO tmp file for template
-    open(my $h, '>', $self->dest)
+sub template_install {
+    my ($template, $dest, $args) = @_;
+
+    # Try to catch some errors
+    if (-e $dest) {
+        die("Destination $dest exists and is not a file")
+          if (-d $dest);
+        die("Destination $dest exists is is not writable")
+          if (-f $dest and ! -w $dest);
+    }
+        
+    # Backup file
+    backup_file($dest)
+      if (-f -e -r $dest);
+
+    debug("Output template: template=<$template> dest=<$dest>");
+
+    open(my $h, '>', $dest) 
       or die("Permission denied writing template");
-    $self->template_file($h);
+    template_file($template, $h, $args);
     close($h);
-    return 1;
 }
 
-sub file_diff {
-    my $self = shift;
-    my $temp_h;
-    # TODO prune args in common function
-    $self->template_file(\$temp_h);
-    say(Text::Diff::diff($self->dest, \$temp_h));
+sub template_diff {
+    my ($template, $dest, $args) = @_;
+
+    # Try to catch some errors
+    if (-e $dest) {
+        die("Destination exists and is not a file: dest=<$dest>")
+          if (-d $dest);
+        die("Destination exists is is not writable: dest=<$dest>")
+          if (-f $dest and ! -w $dest);
+    }
+        
+    if (-f -e -r $dest) {
+        my $temp_h;
+        template_file($template, \$temp_h, $args);
+        say(Text::Diff::diff($dest, \$temp_h));
+    }
 }
 
 # Processing absoulte template name and outputs to reference
 # variable $out. Die on error or unreadable template file
 sub template_file {
-    my ($self, $out) = @_;
+    my $file = shift;
+    my $out = shift;
+    my $args = shift;
     
-    die("Invalid template name")
-      unless($self->src =~ /^[A-Za-z0-9\-\_][A-Za-z0-9\/\-\_\.]+$/);
-    
-    # Build object, finish checks
+    # Create Template object, no config for now
     my $t = Template->new();
-    my $file = get_absolute_template($self->src);
-    $self->{hostname} = Sys::Hostname::Long::hostname_long();
+    
     die("Template not found")
       if(! -e $file);
     die("Permission denied reading template")
       if(! -r $file);
 
-    # Create Template object, no config for now
     open(my $h, $file);
-    $t->process($h, $self, $out) or die $t->error();
+    $t->process($h, $args, $out) or die $t->error();
     close($h);
 }
 
@@ -80,6 +124,7 @@ sub template_file {
 # TODO error checking
 sub get_absolute_template {
     my $template = shift;
+
     return join "/", $FindBin::Bin, $template;
 }
 
